package gridslab;

import java.util.*;
import javax.swing.*;

/**
 * Contains a list of Locations.  Provides methods for performing useful tasks for
 * managing locations.
 *
 * @author Hussein Gibbins.
 */
public class LocationList extends Observable {

	private Vector locations = new Vector();
	private Location removedLocation;
	private Location defaultLocation;

        /**
         * Creates an instance of the LocationList.
         */
	public LocationList() {
		defaultLocation = new Location("", "", 0, "", 0, 0);
	}

	/**
	 * Clears the contents of this list.
	 */
	public void clear() {
		locations.clear();
	}

        /**
         * Adds a location to the front of this list.
         *
         * @param location the location to be added.
         */
	public void add(Location location) {
		locations.add(0, location);
		setChanged();
		notifyObservers("add");
	}

        /**
         * Sets the position in this list given by the index with the location provided.
         *
         * @param index the position in the list to be set.
	 * @param location the location to be set into the list.
         */
	public void set(int index, Location location) {
		locations.set(index, location);
		setChanged();
		notifyObservers("add");
	}

        /**
         * Moves the given location to the front of this list.
         *
         * @param location the location to be moved.
         */
	public void moveToTop(Location location) {
		if (location == locations.firstElement()) {
			return;
		}
		locations.remove(location);
		locations.add(0, location);
		setChanged();
		notifyObservers("update");
	}

        /**
         * Removes the given location from this list.
         *
         * @param location the location to be removed.
         */
	public void remove(Location location) {
		locations.remove(location);
		removedLocation = location;
		setChanged();
		notifyObservers("remove");
	}

        /**
         * Used to notify listeners that a set of updates to this list have completed and
	 * that they should update.
	 *
	 * !used as a workaround for a problem - may nolonger need.
         */
	public void complete() {
		setChanged();
		notifyObservers("complete");
	}

	/**
	 * Used to notify listeners that the list has changed.
	 */
	public void refresh() {
		setChanged();
		notifyObservers();
	}

	/**
	 * Equivalent to the corresponding method of java.util.Vector.  Returns the location
	 * at the given index in this list.
	 *
	 * @param index the index of the location requested.
	 * @return the location specified by the given index.
	 */
	public Location elementAt(int index) {
		return (Location)locations.elementAt(index);
	}
	
        /**
         * Returns a count of the number of locations in this list.
         *
         * @return the number of locations in this list.
         */
	public int size() {
		return locations.size();
	}

	/**
	 * Gets the location at the front of this list.  If none exist yet it will provide a default (empty) location.
	 *
	 * @return a location.
	 */
	public Location getNewLocation() {
		if (locations.size() == 0) {
			return defaultLocation;
		} else {
			return (Location)locations.firstElement();
		}
	}
}

